/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.keycloak;

import java.util.Base64;

import org.keycloak.admin.client.Keycloak;
import org.keycloak.admin.client.KeycloakBuilder;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Scope;

import com.fasterxml.jackson.databind.ObjectMapper;

import io.fabric8.kubernetes.api.model.Secret;
import io.fabric8.kubernetes.client.KubernetesClient;
import io.fabric8.kubernetes.client.dsl.Resource;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Configuration
public class KeycloakClient {

	private final KubernetesClient kubernetesClient;

	@Bean
	@Scope("singleton")
	Keycloak createKeycloak() {
		setProperties();
		return buildKeycloak();
	}

	private void setProperties() {
		System.setProperty("com.sun.xml.ws.transport.http.client.HttpTransportPipe.dump", "true");
		System.setProperty("com.sun.xml.internal.ws.transport.http.client.HttpTransportPipe.dump", "true");
		System.setProperty("com.sun.xml.ws.transport.http.HttpAdapter.dump", "true");
		System.setProperty("com.sun.xml.internal.ws.transport.http.HttpAdapter.dump", "true");
		System.setProperty("com.sun.xml.internal.ws.transport.http.HttpAdapter.dumpTreshold", "999999");
	}

	private Keycloak buildKeycloak() {
		return KeycloakBuilder.builder()
				.serverUrl("http://keycloak-keycloakx-http.keycloak")
				.realm("master")
				.username("admin")
				.password(getKeycloakAdminPassword())
				.clientId("admin-cli")
				.build();
	}

	private String getKeycloakAdminPassword() {
		return decodeBase64(getKeycloakRealmAdminSecret()
				.get()
				.getData()
				.get("password"));
	}

	private Resource<Secret> getKeycloakRealmAdminSecret() {
		return kubernetesClient.secrets()
				.inNamespace("keycloak")
				.withName("keycloak-admin-secret");
	}

	private String decodeBase64(String base64String) {
		return new String(Base64.getDecoder().decode(base64String));
	}

	@Bean
	public ObjectMapper objectMapper() {
		return CustomObjectMapper.createObjectMapper();
	}
}
