/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.keycloak.client;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

import jakarta.ws.rs.core.Response;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.keycloak.admin.client.Keycloak;
import org.keycloak.admin.client.resource.ClientResource;
import org.keycloak.admin.client.resource.ClientsResource;
import org.keycloak.admin.client.resource.RealmResource;
import org.keycloak.admin.client.resource.RoleResource;
import org.keycloak.admin.client.resource.RolesResource;
import org.keycloak.representations.idm.ClientRepresentation;
import org.keycloak.representations.idm.RoleRepresentation;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.operator.keycloak.KeycloakException;

@Disabled("Due to Bugfix")
class KeycloakClientRemoteServiceTest {

	private static final String REALM = "TestNamespace";
	private static final String CLIENT_ID = "TestClientID";

	@Spy
	@InjectMocks
	private KeycloakClientRemoteService service;

	@Mock
	private Keycloak keycloak;
	@Mock
	private RealmResource realmResource;
	@Mock
	private ClientsResource clientsResource;
	@Mock
	private ClientRepresentation clientRepresentation;
	@Mock
	private ClientResource clientResource;
	@Mock
	private Response response;
	@Mock
	private RolesResource rolesResource;
	@Mock
	private RoleRepresentation roleRepresentation;
	@Mock
	private RoleResource roleResource;

	@Nested
	class TestUpdateClient {

		@BeforeEach
		void init() {
			when(keycloak.realm(REALM)).thenReturn(realmResource);
			when(realmResource.clients()).thenReturn(clientsResource);
			when(clientsResource.get(CLIENT_ID)).thenReturn(clientResource);
		}

		@Test
		void shouldUpdateClient() {
			var client = ClientRepresentationTestFactory.create();
			client.setId(CLIENT_ID);

			service.updateClient(client, REALM);

			verify(clientResource).update(client);
		}
	}

	@Nested
	class TestCreateClient {

		@BeforeEach
		void init() {
			when(keycloak.realm(REALM)).thenReturn(realmResource);
			when(realmResource.clients()).thenReturn(clientsResource);
			when(clientsResource.create(clientRepresentation)).thenReturn(response);
		}

		@Test
		void shouldCreateClient() {
			when(response.getStatusInfo()).thenReturn(Response.Status.CREATED);

			service.createClient(clientRepresentation, REALM);

			verify(clientsResource).create(clientRepresentation);
		}

		@Test
		void shouldThrowOnError() {
			when(response.getStatusInfo()).thenReturn(Response.Status.INTERNAL_SERVER_ERROR);

			assertThrows(KeycloakException.class,
					() -> service.createClient(clientRepresentation, REALM));
		}
	}

	@Nested
	class TestClientRoles {

		@Nested
		class TestAddClientRole {

			@BeforeEach
			void init() {
				when(keycloak.realm(REALM)).thenReturn(realmResource);
				when(realmResource.clients()).thenReturn(clientsResource);
				when(clientsResource.get(CLIENT_ID)).thenReturn(clientResource);
				when(clientResource.roles()).thenReturn(rolesResource);
			}

			@Test
			void shouldAddClientRole() {
				service.addClientRole(roleRepresentation, CLIENT_ID, REALM);

				verify(rolesResource).create(roleRepresentation);
			}
		}

		@Nested
		class TestUpdateClientRole {

			private static final String ROLE_ID = "RoleID";

			@BeforeEach
			void init() {
				when(keycloak.realm(REALM)).thenReturn(realmResource);
				when(realmResource.clients()).thenReturn(clientsResource);
				when(clientsResource.get(CLIENT_ID)).thenReturn(clientResource);
				when(clientResource.roles()).thenReturn(rolesResource);
				when(rolesResource.get(ROLE_ID)).thenReturn(roleResource);

				when(roleRepresentation.getName()).thenReturn(ROLE_ID);
			}

			@Test
			void shouldUpdateClientRole() {
				service.updateClientRole(roleRepresentation, CLIENT_ID, REALM);

				verify(roleResource).update(roleRepresentation);
			}
		}
	}
}